#include <stdlib.h>
#include <stdio.h>

#define P (999149) 
#define CARICO (2)

int A, B; // A e B dichiarate qui sono visibili ad ogni funzione 

typedef struct lista_el {

   /* Dato della cella attuale. */
   int dato;
   
   /* Puntatore all'elemento successivo nella lista */
   struct lista_el *succ;
} elemento;

/* Inserisce il valore nella tabella hash di dimensione dim
 * Si utilizza la funzione hash h(valore) = ((A*valore + B) mod P) mod dim 
 * e si risolvono i conflitti utilizzando le liste
 */

void inserisci_elemento(elemento ** tabella, int dim, int valore) {
 
  int valore_hash;
  elemento * tmp;

  valore_hash = ((A*valore + B) % P) % dim;
  tmp = (elemento *) malloc(sizeof(elemento));
  if(tmp == NULL) exit(1);	
  tmp->dato = valore;
  tmp->succ = tabella[valore_hash]; // punterà all'elemento successivo o a NULL
  tabella[valore_hash] = tmp;
}

 /* Versione equivalente */
void inserisci_elemento1(elemento ** tabella, int dim, int valore) {
 
  int valore_hash;
  elemento * tmp;

  valore_hash = ((A*valore + B) % P) % dim;

  if(tabella[valore_hash] == NULL) { 
      // la lista in posizione valore_hash è vuota, ne crea una nuova per memorizzare valore 
      tabella[valore_hash] = (elemento *) malloc(sizeof(elemento));
      if(tabella[valore_hash] == NULL) exit(1);	
      tabella[valore_hash]->dato = valore;
      tabella[valore_hash]->succ = NULL;
  } else { 
      // la lista in posizione valore_hash contiene già altri elementi, inserisci valore in testa
      tmp = tabella[valore_hash]; // memorizza la testa della lista 
      tabella[valore_hash] = (elemento *) malloc(sizeof(elemento)); // crea la nuova testa
      if(tabella[valore_hash] == NULL) exit(1);
      tabella[valore_hash]->dato = valore;
      tabella[valore_hash]->succ = tmp; // collega il nuovo elemento alla vecchia testa
  }
 
}

/* restituisce il numero di elementi in una lista */
int conta_elementi(elemento * testa) {
  
  int num = 0;
  while(testa != NULL){ 
    num++;
    testa = testa->succ;
  }

  return num;
}


int main() {

   elemento ** tabella = NULL;
   int i, n, val, dimensione, valore;
   int conflitti = 0, maxlen = 0, num_elem; // Da utilizzare per calcolare le informazioni

   // Legge il numero di elementi 
   scanf("%d", &n);
   if(n < 1) return 1;

   srand( time(NULL) );
   A = rand()%(100-1)+1;;
   B = rand()%(100-1)+1;;
   // crea la tabella hash 

   dimensione = n * CARICO; // la dimensione della tabella
   tabella = (elemento **) malloc(dimensione * sizeof(elemento *) );
   if(tabella == NULL) exit(1);

   for(i=0; i < dimensione; i++) 
	      tabella[i] = NULL; // ogni lista e' inizialmente vuota 

   // legge gli elementi e li inserisce nella lista
   for(i=0; i < n; i++) {
	scanf("%d", &valore);
	inserisci_elemento(tabella, dimensione, valore);
   }
   
   /* calcola le informazioni sulla tabella */
   for(i=0; i < dimensione; i++) {
      num_elem = conta_elementi(tabella[i]);
      if(maxlen < num_elem) maxlen = num_elem;
      if(num_elem > 1) conflitti += num_elem;
   }
  
   printf("Numero conflitti %d\n", conflitti);
   printf("Lunghezza massima %d\n", maxlen);
   return 0;

}